@@ -8,6 +8,7 @@
 
 #include "gateway_default_handler.h"
 
+#include "gateway_costs.h"
 #include "defs.h"
 #include "gateway.h"
 #include "lq_plugin.h"
@@ -17,12 +18,13 @@ static uint32_t gw_def_stablecount;
 static bool gw_def_choose_new_ipv4_gw;
 static bool gw_def_choose_new_ipv6_gw;
 static struct timer_entry *gw_def_timer;
+static struct costs_weights gw_costs_weights;
 
 /* forward declarations */
 static void gw_default_init(void);
 static void gw_default_cleanup(void);
 static void gw_default_startup_handler(void);
-static uint64_t gw_default_getcosts(struct gateway_entry *gw);
+static int64_t gw_default_getcosts(struct gateway_entry *gw);
 static void gw_default_choosegw_handler(bool ipv4, bool ipv6);
 static void gw_default_update_handler(struct gateway_entry *);
 static void gw_default_delete_handler(struct gateway_entry *);
@@ -50,140 +52,48 @@ struct olsr_gw_handler gw_def_handler = {
  * @param path_cost the path cost
  * @return the threshold path cost
  */
-static inline uint64_t gw_default_calc_threshold(uint64_t path_cost) {
+static inline int64_t gw_default_calc_threshold(int64_t path_cost) {
   if (olsr_cnf->smart_gw_thresh == 0) {
     return path_cost;
   }
 
-  return ((path_cost * (uint64_t) olsr_cnf->smart_gw_thresh) + (uint64_t) 50) / (uint64_t) 100;
-}
-
-/**
- * Weigh the path costs and the gateway bandwidth.
- *
- * If the ETX divider is zero, then no weighing is performed and only the path
- * costs are considered (classic behaviour).
- *
- * If either of the uplink or downlink bandwidths is zero, then UINT64_MAX is
- * returned.
- *
- * @param path_cost the (ETX) path cost to the gateway
- * @param exitUk the gateway exit link uplink bandwidth (in kbps)
- * @param exitDk the gateway exit link downlink bandwidth (in kbps)
- * @return the weighed path cost
- */
-static inline uint64_t gw_default_weigh_costs(uint32_t path_cost, uint32_t exitUk, uint32_t exitDk) {
-  uint8_t WexitU = olsr_cnf->smart_gw_weight_exitlink_up;
-  uint8_t WexitD = olsr_cnf->smart_gw_weight_exitlink_down;
-  uint8_t Wetx = olsr_cnf->smart_gw_weight_etx;
-  uint8_t Detx = olsr_cnf->smart_gw_divider_etx;
-  uint64_t costU;
-  uint64_t costD;
-  uint64_t costE;
-
-  if (!Detx) {
-    /* only consider path costs (classic behaviour) (but scale to 64 bit) */
-    return (uint64_t)path_cost << 32;
-  }
-
-  if (!exitUk || !exitDk) {
-    /* zero bandwidth */
-    return UINT64_MAX;
-  }
-
-  /*
-   * Weighing of the path costs:
-   *
-   * exitUm = the gateway exit link uplink   bandwidth, in Mbps
-   * exitDm = the gateway exit link downlink bandwidth, in Mbps
-   * WexitU = the gateway exit link uplink   bandwidth weight   (configured)
-   * WexitD = the gateway exit link downlink bandwidth weight   (configured)
-   * Wetx   = the ETX path cost weight                          (configured)
-   * Detx   = the ETX path cost divider                         (configured)
-   *
-   *                     WexitU   WexitD   Wetx
-   * path_cost_weight =  ------ + ------ + ---- * path_cost
-   *                     exitUm   exitDm   Detx
-   *
-   * Since the gateway exit link bandwidths are in Kbps, the following formula
-   * is used to convert them to the desired Mbps:
-   *
-   *       bwK
-   * bwM = ----       bwK = bandwidth in Kbps
-   *       1000       bwM = bandwidth in Mbps
-   *
-   * exitUk = the gateway exit link uplink   bandwidth, in Kbps
-   * exitDk = the gateway exit link downlink bandwidth, in Kbps
-   *
-   *                     1000 * WexitU   1000 * WexitD   Wetx
-   * path_cost_weight =  ------------- + ------------- + ---- * path_cost
-   *                         exitUk          exitDk      Detx
-   *
-   *
-   * Analysis of the required bit width of the result:
-   *
-   * exitUk    = [1,   320,000,000] = 29 bits
-   * exitDk    = [1,   320,000,000] = 29 bits
-   * WexitU    = [1,           255] =  8 bits
-   * WexitD    = [1,           255] =  8 bits
-   * Wetx      = [1,           255] =  8 bits
-   * Detx      = [1,           255] =  8 bits
-   * path_cost = [1, 4,294,967,295] = 32 bits
-   *
-   *                         1000 * 255   1000 * 255   255
-   * path_cost_weight(max) = ---------- + ---------- + --- * 4,294,967,295
-   *                              1             1       1
-   *
-   * path_cost_weight(max) = 0x3E418    + 0x3E418    + 0xFEFFFFFF01
-   * path_cost_weight(max) = 0xFF0007C731
-   *
-   * Because we can multiply 0xFF0007C731 by 2^24 without overflowing a
-   * 64 bits number, we do this to increase accuracy.
-   */
-
-  costU = (((uint64_t) (1000 * WexitU)) << 24) / exitUk;
-  costD = (((uint64_t) (1000 * WexitD)) << 24) / exitDk;
-  costE = (((uint64_t) (Wetx * path_cost)) << 24) / Detx;
-
-  return (costU + costD + costE);
+  return ((path_cost * olsr_cnf->smart_gw_thresh) + 50) / 100;
 }
 
 /**
  * Look through the gateway list and select the best gateway
- * depending on the distance to this router
+ * depending on the costs
  */
 static void gw_default_choose_gateway(void) {
-  uint64_t cost_ipv4_threshold = UINT64_MAX;
-  uint64_t cost_ipv6_threshold = UINT64_MAX;
+  int64_t cost_ipv4_threshold = INT64_MAX;
+  int64_t cost_ipv6_threshold = INT64_MAX;
   bool cost_ipv4_threshold_valid = false;
   bool cost_ipv6_threshold_valid = false;
   struct gateway_entry *chosen_gw_ipv4 = NULL;
   struct gateway_entry *chosen_gw_ipv6 = NULL;
-  uint64_t chosen_gw_ipv4_costs = UINT64_MAX;
-  uint64_t chosen_gw_ipv6_costs = UINT64_MAX;
   struct gateway_entry *gw;
   bool dual = false;
 
   if (olsr_cnf->smart_gw_thresh) {
     /* determine the path cost thresholds */
 
-    uint64_t cost = gw_default_getcosts(olsr_get_inet_gateway(false));
-    if (cost != UINT64_MAX) {
-      cost_ipv4_threshold = gw_default_calc_threshold(cost);
+    struct gateway_entry * current_gw = olsr_get_inet_gateway(false);
+    if (current_gw) {
+      cost_ipv4_threshold = gw_default_calc_threshold(current_gw->path_cost);
       cost_ipv4_threshold_valid = true;
     }
 
-    cost = gw_default_getcosts(olsr_get_inet_gateway(true));
-    if (cost != UINT64_MAX) {
-      cost_ipv6_threshold = gw_default_calc_threshold(cost);
+    current_gw = olsr_get_inet_gateway(true);
+    if (current_gw) {
+      cost_ipv6_threshold = gw_default_calc_threshold(current_gw->path_cost);
       cost_ipv6_threshold_valid = true;
     }
   }
 
   OLSR_FOR_ALL_GATEWAY_ENTRIES(gw) {
-    uint64_t gw_cost = gw_default_getcosts(gw);
+    int64_t gw_cost = gw->path_cost;
 
-    if (gw_cost == UINT64_MAX) {
+    if (gw_cost == INT64_MAX) {
       /* never select a node with infinite costs */
       continue;
     }
@@ -192,20 +102,18 @@ static void gw_default_choose_gateway(void) {
       bool gw_eligible_v4 = gw->ipv4
           /* && (olsr_cnf->ip_version == AF_INET || olsr_cnf->use_niit) *//* contained in gw_def_choose_new_ipv4_gw */
           && (olsr_cnf->smart_gw_allow_nat || !gw->ipv4nat);
-      if (gw_eligible_v4 && gw_cost < chosen_gw_ipv4_costs
+      if (gw_eligible_v4 && gw_cost < (chosen_gw_ipv4 ? chosen_gw_ipv4->path_cost : INT64_MAX)
           && (!cost_ipv4_threshold_valid || (gw_cost < cost_ipv4_threshold))) {
         chosen_gw_ipv4 = gw;
-        chosen_gw_ipv4_costs = gw_cost;
       }
     }
 
     if (gw_def_choose_new_ipv6_gw) {
       bool gw_eligible_v6 = gw->ipv6
           /* && olsr_cnf->ip_version == AF_INET6 *//* contained in gw_def_choose_new_ipv6_gw */;
-      if (gw_eligible_v6 && gw_cost < chosen_gw_ipv6_costs
+      if (gw_eligible_v6 && gw_cost < (chosen_gw_ipv6 ? chosen_gw_ipv6->path_cost : INT64_MAX)
           && (!cost_ipv6_threshold_valid || (gw_cost < cost_ipv6_threshold))) {
         chosen_gw_ipv6 = gw;
-        chosen_gw_ipv6_costs = gw_cost;
       }
     }
   } OLSR_FOR_ALL_GATEWAY_ENTRIES_END(gw)
@@ -219,11 +127,11 @@ static void gw_default_choose_gateway(void) {
 
   if (chosen_gw_ipv4) {
     /* we are dealing with an IPv4 or dual stack gateway */
-    olsr_set_inet_gateway(&chosen_gw_ipv4->originator, chosen_gw_ipv4_costs, true, dual);
+    olsr_set_inet_gateway(chosen_gw_ipv4, true, dual);
   }
   if (chosen_gw_ipv6 && !dual) {
     /* we are dealing with an IPv6-only gateway */
-    olsr_set_inet_gateway(&chosen_gw_ipv6->originator, chosen_gw_ipv6_costs, false, true);
+    olsr_set_inet_gateway(chosen_gw_ipv6, false, true);
   }
 
   if ((olsr_cnf->smart_gw_thresh == 0) && !gw_def_choose_new_ipv4_gw && !gw_def_choose_new_ipv6_gw) {
@@ -301,6 +209,11 @@ static void gw_default_init(void) {
   gw_def_choose_new_ipv4_gw = true;
   gw_def_choose_new_ipv6_gw = true;
   gw_def_timer = NULL;
+
+  gw_costs_weights.WexitU = olsr_cnf->smart_gw_weight_exitlink_up;
+  gw_costs_weights.WexitD = olsr_cnf->smart_gw_weight_exitlink_down;
+  gw_costs_weights.Wetx = olsr_cnf->smart_gw_weight_etx;
+  gw_costs_weights.Detx = olsr_cnf->smart_gw_divider_etx;
 }
 
 /**
@@ -335,13 +248,13 @@ static void gw_default_startup_handler(void) {
  * Called when the costs of a gateway must be determined.
  *
  * @param gw the gateway
- * @return the costs, or UINT64_MAX in case the gateway is null or has inifinite costs
+ * @return the costs, or INT64_MAX in case the gateway is null or has infinite costs
  */
-static uint64_t gw_default_getcosts(struct gateway_entry *gw) {
+static int64_t gw_default_getcosts(struct gateway_entry *gw) {
   struct tc_entry* tc;
 
   if (!gw) {
-    return UINT64_MAX;
+    return INT64_MAX;
   }
 
   tc = olsr_lookup_tc_entry(&gw->originator);
@@ -350,11 +263,11 @@ static uint64_t gw_default_getcosts(struct gateway_entry *gw) {
     /* gateways should not exist without tc entry */
     /* do not consider nodes with an infinite ETX */
     /* do not consider nodes without bandwidth or with a uni-directional link */
-    return UINT64_MAX;
+    return INT64_MAX;
   }
 
   /* determine the path cost */
-  return gw_default_weigh_costs(tc->path_cost, gw->uplink, gw->downlink);
+  return gw_costs_weigh(true, gw_costs_weights, tc->path_cost, gw->uplink, gw->downlink);
 }
 
 /**
